#!perl

use 5.006;
use strict;
use warnings;

# this test was generated with
# Dist::Zilla::Plugin::Author::SKIRMESS::RepositoryBase 0.017

use Test::MinimumVersion 0.008;

all_minimum_version_from_metayml_ok();
